#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <glob.h>
#include <unistd.h>
#include <getopt.h>
#include "pvo.hh"
#include "pvo-block.hh"
#include "pvo-header.hh"
#include "pvo-utils.hh"
using namespace PVO;
using namespace PVO::Header;
using namespace PVO::Block;
using std::string;

int main(int argc, char * argv[])
{
  /** 
   * S70Summary is a function to print out basic information for
   * 70cm survey tape files
   *
   * This program is a command-line program that looks at existing
   * survey files and returns a summary of their starting positions, 
   * observation times and moving flags
   *
   * The format of usage is: <br> 
   * <tt>> pvotapesummary </tt><br>
   * to analyse all files in the current directory, or <br>
   * <tt>> pvotapesummary -d DIR</tt><br>
   * to analyse all files in directory DIR
   * Entering<br>
   * <tt>> pvotapesummary -h</tt><br>
   * will give a help message, using PVO::pvotapesummary_usage_text.
   *
   * The output is a list of valid files, the central RA & DEC of the
   * first block, the central observing frequency, the requested scan
   * length, the time/date of observation, and the moving flag.
   * 
   */

  int c;
  unsigned int i;
  glob_t globbuf;
  std::vector<std::string> namelist;
  std::string directory,filenames;
  while (true) {
    int option_index = 0;
    static struct option long_options[] = {
      {"help", 0, 0, 'h'},
      {0, 0, 0, 0}
    };

    c = getopt_long (argc, argv, "hd:",
		     long_options, &option_index);

    if (c == -1) break;
    
    switch (c) {               
    case 0:
      // This is where we deal with -- options that don't have a
      // single letter equivalent
      break;
    case 'h': // give help statement
      std::cout << pvotapesummary_usage_text;
      return 0;
      break;
    case 'd':
      directory = optarg;
      filenames = directory + "/*";
      globbuf.gl_offs = 0;
      glob(filenames.c_str(), GLOB_DOOFFS, NULL, &globbuf);
      namelist.resize(globbuf.gl_pathc);
      for(i=0;i<globbuf.gl_pathc+globbuf.gl_offs;i++) {
	namelist[i] = globbuf.gl_pathv[i];
      }
      break;
    case '?': // invalid option
    case ':': // missing an argument
      std::cout << pvotapesummary_usage_text;
      return 1;
      break;
    default:
      std::cerr << "ERROR: getopt returned character code " <<  c << "\n";
      break;
    }

  }
  
  while (optind < argc) { 
    // There are non-option arguments -- must be the files to look at
     namelist.push_back(argv[optind++]);
  }

  std::vector<std::string> goodlist;
  for(i=0; i<namelist.size(); i++)
    if(idSurvey(namelist[i]) != DUD) goodlist.push_back(namelist[i]);
  
  int filenameSize = 8; // size of "FILENAME"
  for(i=0;i<goodlist.size();i++) 
    filenameSize = std::max(filenameSize,int(goodlist[i].size()));
    
  std::cout << std::setfill('-')  
	    << std::setw(filenameSize+1+15+15+31+11+11+5) << "-";

  std::cout << "\n" << std::setfill(' ') 
	    << std::setw(filenameSize) << "FILENAME"  << " "
	    << std::setw(14)           << "RAJ"       << " "
	    << std::setw(14)           << "DECJ"      << " "
	    << std::setw(10)           << "OBSFREQ"   << " "
	    << std::setw(10)           << "SCANLEN"   << " "
	    << std::setw(4)            << "MOVE"  << " "
	    << std::setw(30)           << "DATE-OBS\n";
     
  std::cout << std::setw(filenameSize) << ""      << " "
	    << std::setw(14)           << ""      << " "
	    << std::setw(14)           << ""      << " "
	    << std::setw(10)           << "[MHz]" << " "
	    << std::setw(10)           << "[s]"   << " "
	    << std::setw(4)            << ""      <<" "
	    << std::setw(30)           << "\n";

  std::cout<< std::setfill('-')  
	   << std::setw(filenameSize+1+15+15+31+11+11+5) << "-";
  std::cout<< "\n"<< std::setfill(' ') ;
    
  for(i=0;i<goodlist.size();i++){

    std::ifstream fin;
    fin.open(goodlist[i].c_str(), std::ios::binary);
    DataBlock block(S70);
    block.readBlock(fin);
    std::string ra       = deblank(block.getKeyword<std::string>(RA));
    std::string dec      = deblank(block.getKeyword<std::string>(DEC));
    float scanlen        = block.getKeyword<float>(SCAN_LEN);
    int move             = block.getKeyword<int>(MOVE);
    std::string date_obs = 
      makeDateObs(deblank(block.getKeyword<std::string>(DATE)),
		  deblank(block.getKeyword<std::string>(UT))   );
    double f0            = block.getKeyword<double>(FREQ_CHAN1);
    double bw            = block.getKeyword<double>(CHANBW);
    double nc            = block.getKeyword<double>(NCHAN);
    float obsfreq        = f0 + ((nc-1)/2.) * bw;

    std::cout.setf(std::ios::fixed);
    std::cout << std::setw(filenameSize) << goodlist[i] << " "
	      << std::setw(14)           << ra          << " "
	      << std::setw(14)           << dec         << " "
	      << std::setprecision(4)
	      << std::setw(10)           << obsfreq     << " "
	      << std::setw(10)           << scanlen     << " "
	      << std::setw(4)            << move        << " " 
	      << std::setw(30)           << date_obs    << "\n";

    fin.close();
  }

}

